import gc
import micropython
import lvgl as lv
import time
import sys
from espidf import VSPI_HOST
from ili9XXX import ili9488
from xpt2046 import xpt2046 
from machine import Pin, I2C, PWM, Timer
import os
import machine
import array
import pid_aw as PID
#import pid_aw
from pid_aw import *
import uctypes
from uctypes import BF_POS, BF_LEN, BFUINT8,UINT8,FLOAT32


i2c = machine.I2C(scl=machine.Pin(21), sda=machine.Pin(22))
import DFRobot_MAX31855
max31855 = DFRobot_MAX31855.DFRobot_MAX31855(i2c)
disp = ili9488(miso=12, mosi=13, clk=14, cs=15, dc=2, rst=-1,backlight=27, backlight_on=1, power_on=1,
               spihost=VSPI_HOST, mhz=20,power=-1,
               factor=16, hybrid=True, width=320, height=480,
               invert=False, double_buffer=True, half_duplex=False)

touch = xpt2046(cs=33, spihost=VSPI_HOST, mosi=-1, miso=-1, clk=-1, cal_x0 = 340 ,cal_x1 = 3700 ,cal_y0 = 3700, cal_y1 = 340)

#
####
#

#
###  def variables and init values
#
temperature = 50 
temp = 50.
req_temp = 50                  # pozadovana teplota
req_time = 10                  # pozadovany cas
btn_set = 1                    # sada prednasaveni 
min_max = 60
is_start = 0                   # bezi rezim  start
is_stop = 0                    # bezi stop
run_manual = 0                 # heating in manual state
is_manual = 0                  # je zvoleny rezim manual
r_temp = []                    # zoznam pozadovanych teplôt
r_time = []                    # zoznam pozadovanych casov v sekundach
a_time = 0                     # actually requested heat time
u = 0
u_man = 0                      # output value in manual mode
sp = 0.                        # actually requested heat temperature by PID
sp_m = 50.                     # actually requested heat temperature by PID for manual mode
pv = 0.                        # actually measured temperature by PID
h_text = ""                    # text horneho riadku
duty = 1023                    # value for heating off
x_range = 0                    # set chart_range
r_set = 0                      # refresh requested
x_time2 = 8000                 # plot interval
cnt_i = 0                      # helper counter
cnt_run = 0                    # helper counter
cnt_err = 0                    # helper counter
is_OK = 0                      # button OK was pressed
is_cancel = 0                  # button cancel was presset
r_man = 0                      # running set values for manual imput
is_error = 0                   # system error has been appeared
is_plot = 0                    # plot running
err_code = ""                  # fatal error code text
delta_t = 0.                    # temperature increment during regulation loop
temp_b = 0.                     # temperature measured in previous loop
#
scr = lv.obj()  # scr====> screen 
scr = lv.scr_act()
scr.clean()

#
###  header definition
#
l_header = lv.label(scr)
l_header.set_text(h_text)
l_header.align(lv.ALIGN.TOP_LEFT,5,5)
#l_header.set_x() 
#l_header.set_y()


#
###     button definition
#

btn_start = lv.btn(scr)  
btn_start.set_size(100, 40)
btn_start.align(lv.ALIGN.BOTTOM_LEFT,5,-10)
btn_start.add_style(lv.style_t(), 0)
btn_start.add_style(lv.style_t(), lv.STATE.PRESSED)
cnt_start = 0
def btn_event_start(evt):
    beep(freq=4186, time_b=0.01, duty=50)
    code_start = evt.get_code()
    global is_start, ser1, ser2, is_stop, x_time2, btn_set,h_text,r_temp,r_time,x_range, r_set, r_man, is_manual, is_plot, a_time,cnt_err, u_man, run_manual, cnt_i
    is_stop = 0
    btn_start.set_style_bg_color(lv.palette_main(lv.PALETTE.GREEN),0)
    btn_stop.set_style_bg_color(lv.palette_main(lv.PALETTE.BLUE),0)
    btn_stop.set_style_bg_color(lv.palette_main(lv.PALETTE.BLUE),0)
    if code_start == lv.EVENT.CLICKED:
        l_info.add_flag(lv.obj.FLAG.HIDDEN)
        t_info.add_flag(lv.obj.FLAG.HIDDEN)
        is_stop = r_man = 0
        r_set = 1
        is_start = 1
        is_plot = 1
        x_time2 = 0
        print("Start button pressed")
        if is_start == 0:
            cnt_i = 0
            a_time = r_time[0]
        if is_manual == 0:
            chart.remove_series(ser2)
            ser2 = chart.add_series(lv.palette_main(lv.PALETTE.GREEN), lv.chart.AXIS.SECONDARY_Y)
            btn_set,h_text,r_temp,r_time,x_range = read_set(btn_set,h_text,r_temp,r_time,x_range)
            chart.set_range(lv.chart.AXIS.PRIMARY_X, 0, x_range)
            chart.set_range(lv.chart.AXIS.SECONDARY_X, 0, x_range)
            chart.refresh()
        else:
            run_manual = 1
        if is_start != 1:
           x_time2 = x_range
           a_time = 0
           cnt_err = 0
           u_man = 0
        else:
            cnt_run = 0
            if is_manual == 0:
               PID1.Kp = 3000                 # PID controller parameters for non manual 3000
               PID1.Td = 1000                 # 1000
               PID1.Ti = 0.0001               # 0.0001
               PID1.Tt = PID1.Ts              # 0.1
               PID1.Umin = 0.
               PID1.Umax = 1023.
               pid_tune(PID1)

btn_start.add_event(btn_event_start, lv.EVENT.CLICKED, None)  

label_start = lv.label(btn_start)
label_start.set_style_text_font(lv.font_montserrat_16,0)
label_start.set_text(lv.SYMBOL.PLAY)  
label_start.center()  


btn_stop = lv.btn(scr)  
btn_stop.set_size(100, 40)
btn_stop.align(lv.ALIGN.BOTTOM_MID, 0,-10)
btn_stop.add_style(lv.style_t(), 0)
btn_stop.add_style(lv.style_t(), lv.STATE.PRESSED)
cnt_stop = 0
def btn_event_stop(evt):
    beep(freq=4186, time_b=0.01, duty=50)
    code_stop = evt.get_code()
    global is_stop, is_start, r_set, is_manual, r_man, x_time2, x_range, cnt_i, is_plot, a_time,u_man
    is_stop=1
    if code_stop == lv.EVENT.CLICKED:
        l_info.add_flag(lv.obj.FLAG.HIDDEN)
        t_info.add_flag(lv.obj.FLAG.HIDDEN)
        is_stop = 1
        is_start = 0
        is_plot = 0
        cnt_i = 0
        cnt_run = 0
        u_man = 0
        a_time = is_start = r_man = 0
        print("Stop button pressed")
        btn_start.set_style_bg_color(lv.palette_main(lv.PALETTE.BLUE),0)
        btn_stop.set_style_bg_color(lv.palette_main(lv.PALETTE.RED),0)

btn_stop.add_event(btn_event_stop, lv.EVENT.CLICKED, None)  

label_stop = lv.label(btn_stop)  
label_stop.set_text(lv.SYMBOL.STOP)
label_stop.center()  

btn_manual = lv.btn(scr)  
btn_manual.set_size(100, 40)
btn_manual.add_style(lv.style_t(), 0)
btn_manual.add_style(lv.style_t(), lv.STATE.PRESSED)
btn_manual.align(lv.ALIGN.BOTTOM_RIGHT, -5,-10)  
cnt_manual = 0
def btn_event_manual(evt):
    beep(freq=4186, time_b=0.01, duty=50)
    global is_manual, r_man, is_start, r_set , is_plot, run_manual, sp, cnt_run
    code_manual = evt.get_code()
    if code_manual == lv.EVENT.CLICKED and is_start == 0:
        l_info.add_flag(lv.obj.FLAG.HIDDEN)
        t_info.add_flag(lv.obj.FLAG.HIDDEN)
        print("Manual button pressed")
        btn_stop.set_style_bg_color(lv.palette_main(lv.PALETTE.BLUE),0)
        btn_manual.set_style_bg_color(lv.palette_main(lv.PALETTE.GREEN),0)
        btn_set1.set_style_bg_color(lv.palette_main(lv.PALETTE.BLUE),0)
        btn_set2.set_style_bg_color(lv.palette_main(lv.PALETTE.BLUE),0)
        btn_set3.set_style_bg_color(lv.palette_main(lv.PALETTE.BLUE),0)
        chart.add_flag(lv.obj.FLAG.HIDDEN)
        label_Y_axis.add_flag(lv.obj.FLAG.HIDDEN)
        label_X_axis.add_flag(lv.obj.FLAG.HIDDEN)
        label_meter.clear_flag(lv.obj.FLAG.HIDDEN)
        meter.clear_flag(lv.obj.FLAG.HIDDEN)
        h_text=" Set temperature:  "+ str (req_temp) + "°C   time: "+str(req_time)+ "sec."
        l_header.set_text(h_text)
        #label_manual.set_text("Manual: " + str(cnt_manual))   
        r_set = 0
        is_plot = 0
        u_man = 0
        if is_start == 0:
           PID1.Kp = 3.                  # PID contorller parameters for manual settings
           PID1.Td = 1.0
           
           PID1.Ti = 0.0001
           PID1.Tt = PID1.Ts            # 0.1
           PID1.Umin = -1023.
           PID1.Umax = 1023.
           pid_tune(PID1)
           run_manual = 1
           sp_m = 50
           cnt_run = 0
           if is_manual == 1:
               r_man = 1    
           is_manual = 1
btn_manual.add_event(btn_event_manual, lv.EVENT.CLICKED, None)

label_manual = lv.label(btn_manual)
label_manual.set_style_text_font(lv.font_montserrat_16,0)
label_manual.set_text(lv.SYMBOL.SETTINGS)  
label_manual.center()



btn_set1 = lv.btn(scr)  
btn_set1.set_size(100, 40)
btn_set1.align(lv.ALIGN.BOTTOM_LEFT,5,-60)
btn_set1.add_style(lv.style_t(), 0)
btn_set1.add_style(lv.style_t(), lv.STATE.PRESSED)
def btn_event_set1(evt):
    l_info.add_flag(lv.obj.FLAG.HIDDEN)
    t_info.add_flag(lv.obj.FLAG.HIDDEN)
    beep(freq=4186, time_b=0.01, duty=50)
    code_set1 = evt.get_code()
    global btn_set, is_manual, r_man, r_set
    global r_temp                     # zoznam pozadovanych teplôt
    global r_time                     # zoznam pozadovanych casov v sekundach
    global h_text                     # text horneho riadku
    global x_range                    # chart x-axis range
    if code_set1 == lv.EVENT.CLICKED and is_start == 0 :
        btn_set1.set_style_bg_color(lv.palette_main(lv.PALETTE.GREEN),0)
        btn_set2.set_style_bg_color(lv.palette_main(lv.PALETTE.BLUE),0)
        btn_set3.set_style_bg_color(lv.palette_main(lv.PALETTE.BLUE),0)
        btn_stop.set_style_bg_color(lv.palette_main(lv.PALETTE.BLUE),0)
        btn_manual.set_style_bg_color(lv.palette_main(lv.PALETTE.BLUE),0)
        print("Set1 button pressed")
        is_manual = 0
        r_man = 0
        r_set = 1
        btn_set = 1
        btn_set,h_text,r_temp,r_time,x_range = read_set(btn_set,h_text,r_temp,r_time,x_range)
        chart.set_range(lv.chart.AXIS.PRIMARY_X, 0, x_range)
        chart.set_range(lv.chart.AXIS.SECONDARY_X, 0, x_range)
        l_header.set_text(h_text)
        chart.clear_flag(lv.obj.FLAG.HIDDEN)
        meter.add_flag(lv.obj.FLAG.HIDDEN)
        label_Y_axis.clear_flag(lv.obj.FLAG.HIDDEN)
        label_X_axis.clear_flag(lv.obj.FLAG.HIDDEN)
        label_meter.add_flag(lv.obj.FLAG.HIDDEN)
        #label_start.set_text("Start: " + str(cnt_start))   

btn_set1.add_event(btn_event_set1, lv.EVENT.CLICKED, None)  

label_set1 = lv.label(btn_set1)  
label_set1.set_text("Set 1")
label_set1.set_style_text_font(lv.font_montserrat_16,0)
btn_set1.set_style_bg_color(lv.palette_main(lv.PALETTE.GREEN),0)
label_set1.center()

btn_set2 = lv.btn(scr)  
btn_set2.set_size(100, 40)
btn_set2.align(lv.ALIGN.BOTTOM_MID, 0,-60)
btn_set2.add_style(lv.style_t(), 0)
btn_set2.add_style(lv.style_t(), lv.STATE.PRESSED)
def btn_event_set2(evt):
    l_info.add_flag(lv.obj.FLAG.HIDDEN)
    t_info.add_flag(lv.obj.FLAG.HIDDEN)
    beep(freq=4186, time_b=0.01, duty=50)
    code_set2 = evt.get_code()
    global btn_set, r_man, is_manual, r_set
    global r_temp                     # zoznam pozadovanych teplôt
    global r_time                     # zoznam pozadovanych casov v sekundach
    global h_text                     # text horneho riadku
    global x_range                    # chart x-axis range
    if code_set2 == lv.EVENT.CLICKED and is_start == 0:
        btn_set1.set_style_bg_color(lv.palette_main(lv.PALETTE.BLUE),0)
        btn_set2.set_style_bg_color(lv.palette_main(lv.PALETTE.GREEN),0)
        btn_set3.set_style_bg_color(lv.palette_main(lv.PALETTE.BLUE),0)
        btn_stop.set_style_bg_color(lv.palette_main(lv.PALETTE.BLUE),0)
        btn_manual.set_style_bg_color(lv.palette_main(lv.PALETTE.BLUE),0)
        print("Set2 button pressed")
        is_manual = 0
        r_man = 0
        r_set = 1
        btn_set = 2
        btn_set,h_text,r_temp,r_time,x_range = read_set(btn_set,h_text,r_temp,r_time,x_range)
        chart.set_range(lv.chart.AXIS.PRIMARY_X, 0, x_range)
        chart.set_range(lv.chart.AXIS.SECONDARY_X, 0, x_range)
        chart.clear_flag(lv.obj.FLAG.HIDDEN)
        meter.add_flag(lv.obj.FLAG.HIDDEN)
        label_Y_axis.clear_flag(lv.obj.FLAG.HIDDEN)
        label_X_axis.clear_flag(lv.obj.FLAG.HIDDEN)
        label_meter.add_flag(lv.obj.FLAG.HIDDEN)
        l_header.set_text(h_text)

btn_set2.add_event(btn_event_set2, lv.EVENT.CLICKED, None)  

label_set2 = lv.label(btn_set2)  
label_set2.set_text("Set 2")
label_set2.set_style_text_font(lv.font_montserrat_16,0)
label_set2.center()



btn_set3 = lv.btn(scr)  
btn_set3.set_size(100, 40)
btn_set3.align(lv.ALIGN.BOTTOM_RIGHT, -5,-60)
btn_set3.add_style(lv.style_t(), 0)
btn_set3.add_style(lv.style_t(), lv.STATE.PRESSED)
def btn_event_set3(evt):
    l_info.add_flag(lv.obj.FLAG.HIDDEN)
    t_info.add_flag(lv.obj.FLAG.HIDDEN)
    beep(freq=4186, time_b=0.01, duty=50)
    code_set3 = evt.get_code()
    global btn_set, r_man, is_manual, r_set
    global r_temp                     # zoznam pozadovanych teplôt
    global r_time                     # zoznam pozadovanych casov v sekundach
    global h_text                     # text horneho riadku
    global x_range                    # chart x-axis range
    if code_set3 == lv.EVENT.CLICKED and is_start == 0:
        btn_set1.set_style_bg_color(lv.palette_main(lv.PALETTE.BLUE),0)
        btn_set2.set_style_bg_color(lv.palette_main(lv.PALETTE.BLUE),0)
        btn_set3.set_style_bg_color(lv.palette_main(lv.PALETTE.GREEN),0)
        btn_stop.set_style_bg_color(lv.palette_main(lv.PALETTE.BLUE),0)
        btn_manual.set_style_bg_color(lv.palette_main(lv.PALETTE.BLUE),0)
        print("Set1 button pressed")
        is_manual = 0
        r_man = 0
        r_set = 1
        btn_set = 3
        btn_set,h_text,r_temp,r_time,x_range = read_set(btn_set,h_text,r_temp,r_time,x_range)
        chart.set_range(lv.chart.AXIS.PRIMARY_X, 0, x_range)
        chart.set_range(lv.chart.AXIS.SECONDARY_X, 0, x_range)
        l_header.set_text(h_text)
        chart.clear_flag(lv.obj.FLAG.HIDDEN)
        label_Y_axis.clear_flag(lv.obj.FLAG.HIDDEN)
        label_X_axis.clear_flag(lv.obj.FLAG.HIDDEN)
        meter.add_flag(lv.obj.FLAG.HIDDEN)
        label_meter.add_flag(lv.obj.FLAG.HIDDEN)
        #label_start.set_text("Start: " + str(cnt_start))   

btn_set3.add_event(btn_event_set3, lv.EVENT.CLICKED, None)  

label_set3 = lv.label(btn_set3)  
label_set3.set_text("Set 3")
label_set3.set_style_text_font(lv.font_montserrat_16,0)
label_set3.center()



btn_OK = lv.btn(scr)  
btn_OK.set_size(100, 40)
btn_OK.add_style(lv.style_t(), 0)
btn_OK.add_style(lv.style_t(), lv.STATE.PRESSED)
btn_OK.set_style_bg_color(lv.palette_main(lv.PALETTE.GREEN),0)
btn_OK.add_flag(lv.obj.FLAG.HIDDEN)
btn_OK.align(lv.ALIGN.BOTTOM_RIGHT, -55,-45)  
cnt_OK = 0
def btn_event_OK(evt):
    l_info.add_flag(lv.obj.FLAG.HIDDEN)
    t_info.add_flag(lv.obj.FLAG.HIDDEN)
    global is_OK
    code_OK = evt.get_code()
    if code_OK == lv.EVENT.CLICKED and is_start == 0:
        beep(freq=4186, time_b=0.01, duty=50)
        print("OK button pressed")
        is_OK = 1
        
btn_OK.add_event(btn_event_OK, lv.EVENT.CLICKED, None)

label_OK = lv.label(btn_OK)
label_OK.set_style_text_font(lv.font_montserrat_16,0)
label_OK.set_text(lv.SYMBOL.OK)  
label_OK.center()



btn_cancel = lv.btn(scr)  
btn_cancel.set_size(100, 40)
btn_cancel.add_style(lv.style_t(), 0)
btn_cancel.align(lv.ALIGN.BOTTOM_LEFT,55,-45)
btn_cancel.set_style_bg_color(lv.palette_main(lv.PALETTE.RED),0)
btn_cancel.add_flag(lv.obj.FLAG.HIDDEN)
#label_cancel = lv.label(btn_cancel)
#btn.align(lv.ALIGN.CENTER,-100,0)
cnt_cancel = 0
def btn_event_cancel(evt):
    global is_cancel
    code_cancel = evt.get_code()
    if code_cancel == lv.EVENT.CLICKED and is_start == 0:
        beep(freq=4186, time_b=0.01, duty=50)
        print("Cancel button pressed")
        is_cancel = 1
        #label_manual.set_text("Manual: " + str(cnt_manual))   

btn_cancel.add_event(btn_event_cancel, lv.EVENT.CLICKED, None)

label_cancel = lv.label(btn_cancel)
label_cancel.set_style_text_font(lv.font_montserrat_16,0)
label_cancel.set_text(lv.SYMBOL.CLOSE)  
label_cancel.center()


#
### read curve settings file from setx.txt
#
def read_set(btn_set,h_text,r_temp,r_time,x_range):

 r_time = r_temp = []
 h_text = ""
 x_range = 0
 file = "set"+str(btn_set)+".txt"
 with open(file,'r') as f:
        print("Read file "+ file + " With settings" )
        record = f.readline()                         # read first line
        h_text = f.readline()                         # read seccond line
        f.readline()
        line = f.readline()
        r_temp = line.split(",")
        line = f.readline()
        r_time = line.split(",")
        f.close()
        for i in range (0,len(r_time)):
             x_range = x_range + int(r_time[i])
        if x_range <= 100:
                 x_range = 100
        elif x_range <= 200:    
                 x_range = 200
        elif x_range <= 400:    
                 x_range = 400
        elif x_range <= 800:    
                 x_range = 800         
        elif x_range <= 1000:    
                 x_range = 1000
        elif x_range <= 2000:    
                 x_range = 2000
        elif x_range <= 4000:    
                 x_range = 4000
        elif x_range <= 800:    
                 x_range = 8000
        elif x_range <= 10000:    
                 x_range = 10000
        elif x_range <= 20000:    
                 x_range = 20000
        else:
                 x_range = 40000     
 return(btn_set,h_text,r_temp,r_time,x_range)    



# draw line on top of screen
# Create an object with the new style
line1 = lv.line(lv.scr_act())
line1.add_style(lv.style_t(), 1)
p =  [ {"x":0, "y":30}, 
       {"x":320, "y":30}]

line1.set_points(p, 2)

line1.align_to(lv.scr_act(),lv.ALIGN.TOP_MID,0,0)

#
###   meter object def
#


    #
    # A simple meter
    #

meter = lv.meter(lv.scr_act())
meter.center()
meter.set_scale_range(50, 250,260,140)
meter.set_size(250, 250)
meter.add_flag(lv.obj.FLAG.HIDDEN)
meter.align(lv.ALIGN.CENTER, 0, -45)

# Add a scale first
scale = meter.add_scale_lines(lv.palette_main(lv.PALETTE.GREEN),lv.palette_main(lv.PALETTE.GREEN),False,0 )
meter.set_scale_ticks(81, 50, 12, lv.palette_main(lv.PALETTE.GREY))
meter.set_scale_major_ticks(10, 3, 16, lv.color_black(), 16)

indic = lv.meter_indicator_t()

# Add a blue arc to the start
indic = meter.add_arc(3, lv.palette_main(lv.PALETTE.GREEN), 0)
meter.set_indicator_start_value(indic, 50)
meter.set_indicator_end_value(indic, 80)

# Make the tick lines blue at the start of the scale
indic = meter.add_scale_lines(lv.palette_main(lv.PALETTE.GREEN), lv.palette_main(lv.PALETTE.GREEN), False, 0)
meter.set_indicator_start_value(indic, 50)
meter.set_indicator_end_value(indic, 80)

# Add a red arc to the end
indic = meter.add_arc(3, lv.palette_main(lv.PALETTE.RED), 0)
meter.set_indicator_start_value(indic, 180)
meter.set_indicator_end_value(indic, 280)

# Make the tick lines red at the end of the scale
indic = meter.add_scale_lines(lv.palette_main(lv.PALETTE.RED), lv.palette_main(lv.PALETTE.RED), False, 0)
meter.set_indicator_start_value(indic, 180)
meter.set_indicator_end_value(indic, 250)

# Add a needle line indicator
indic_temp = meter.add_needle_line(4, lv.palette_main(lv.PALETTE.NONE), -10)
meter.set_indicator_value(indic_temp,temperature)

label_meter = lv.label(scr)
label_meter.set_text(str(int(temp))+"°C")
label_meter.set_style_text_font(lv.font_montserrat_16,0)
label_meter.align(lv.ALIGN.CENTER, 0, 40)

#
###
#
# Create a chart

btn_set,h_text,r_temp,r_time,x_range = read_set(btn_set,h_text,r_temp,r_time,x_range)
l_header.set_text(h_text)


chart = lv.chart(lv.scr_act())
chart.set_size(275, 300)
chart.align(lv.ALIGN.CENTER, 17, -45)
chart.set_range(lv.chart.AXIS.PRIMARY_Y, 50, 250)
chart.set_range(lv.chart.AXIS.SECONDARY_Y, 50, 250)
chart.set_range(lv.chart.AXIS.PRIMARY_X, 0, x_range)
chart.set_range(lv.chart.AXIS.SECONDARY_X, 0, x_range)
chart.set_axis_tick(lv.chart.AXIS.PRIMARY_Y, 10, 5, 5, 5, True, 40)
chart.set_axis_tick(lv.chart.AXIS.PRIMARY_X, 10, 6, 5, 6, True, 50)

label_Y_axis = lv.label(scr)
label_Y_axis.set_text("°C")  
label_Y_axis.align(lv.ALIGN.TOP_LEFT, 25, 35)
label_X_axis = lv.label(scr)
label_X_axis.set_text("sec.")  
label_X_axis.align(lv.ALIGN.BOTTOM_RIGHT, -15, -145)


#chart.center()
chart.set_type(lv.chart.TYPE.SCATTER)   # Show lines and points too
chart.set_style_size( 0, 0, lv.PART.INDICATOR )

# Add two data series
ser1 = chart.add_series(lv.palette_main(lv.PALETTE.RED), lv.chart.AXIS.PRIMARY_Y)
ser2 = chart.add_series(lv.palette_main(lv.PALETTE.GREEN), lv.chart.AXIS.SECONDARY_Y)
x_time=0
chart.set_next_value2(ser1, x_time, 50)
for i in range(0,len(r_time)):
    x_time = x_time + int(r_time[i])
    chart.set_next_value2(ser1, x_time, int(r_temp[i]))
    
chart.set_point_count(x_range + 1)    
chart.refresh()      #  Required after direct set

# Create an Arc
###   function for insert  temperature and time value for manual regime
#
arc = lv.arc(lv.scr_act())
arc.set_end_angle(135)
arc.set_size(250, 250)
arc.set_bg_angles(135,45)
arc.set_range(50,250)
arc.center()
arc.add_flag(lv.obj.FLAG.HIDDEN)
arc_label = lv.label(lv.scr_act())
arc_label.set_text('')
arc_label.set_style_text_color(lv.color_hex(0xffffff), lv.PART.MAIN)
arc.set_value(10)
arc.align_obj_to_angle(arc_label,0)

l_arc = lv.label(scr)
l_arc.set_text("")  
l_arc.align(lv.ALIGN.CENTER, 0, -185)
l_arc.add_flag(lv.obj.FLAG.HIDDEN)

def get_man_value(req_time, req_temp):
    global is_OK, is_cancel, h_text, r_man
    l_arc.clear_flag(lv.obj.FLAG.HIDDEN)
    l_arc.set_style_text_font(lv.font_montserrat_16,0)
    btn_OK.clear_flag(lv.obj.FLAG.HIDDEN)
    btn_cancel.clear_flag(lv.obj.FLAG.HIDDEN)
    arc.clear_flag(lv.obj.FLAG.HIDDEN)
    chart.add_flag(lv.obj.FLAG.HIDDEN)
    meter.add_flag(lv.obj.FLAG.HIDDEN)
    btn_set1.add_flag(lv.obj.FLAG.HIDDEN)
    btn_set2.add_flag(lv.obj.FLAG.HIDDEN)
    btn_set3.add_flag(lv.obj.FLAG.HIDDEN)
    btn_manual.add_flag(lv.obj.FLAG.HIDDEN)
    btn_start.add_flag(lv.obj.FLAG.HIDDEN)
    btn_stop.add_flag(lv.obj.FLAG.HIDDEN)
    data = req_temp
    arc.set_value(int(req_temp))
    is_OK = is_cancel = 0
    while (is_OK + is_cancel) == 0:
        arc.set_range(50,250)
        data = get_value(data)
        l_arc.set_text("  Temperature in °C: " + str(data))
    if is_OK == 1:
       req_temp = data
    is_OK = is_cancel = 0
    data = req_time
    arc.set_value(int(req_time))
    while (is_OK + is_cancel) == 0:
        arc.set_range(10,600)
        data = get_value(data)
        l_arc.set_text("  Time in seconds :" + str(data))
    if is_OK == 1:
       req_time = data
    arc_label.set_text('')
    meter.clear_flag(lv.obj.FLAG.HIDDEN)
    label_meter.clear_flag(lv.obj.FLAG.HIDDEN)
    btn_set1.clear_flag(lv.obj.FLAG.HIDDEN)
    btn_set2.clear_flag(lv.obj.FLAG.HIDDEN)
    btn_set3.clear_flag(lv.obj.FLAG.HIDDEN)
    btn_manual.clear_flag(lv.obj.FLAG.HIDDEN)
    btn_start.clear_flag(lv.obj.FLAG.HIDDEN)
    btn_stop.clear_flag(lv.obj.FLAG.HIDDEN)
    arc.add_flag(lv.obj.FLAG.HIDDEN)
    btn_OK.add_flag(lv.obj.FLAG.HIDDEN)
    btn_cancel.add_flag(lv.obj.FLAG.HIDDEN)
    l_arc.add_flag(lv.obj.FLAG.HIDDEN)
    h_text=" Set temperature:  "+ str (req_temp) + "°C   time: "+str(req_time)+ "sec."
    l_header.set_text(h_text)
    is_OK = is_cancel = r_man = 0
    return(req_time, req_temp)

def get_value(data):
    meter.add_flag(lv.obj.FLAG.HIDDEN)
    label_meter.add_flag(lv.obj.FLAG.HIDDEN)
    arc.clear_flag(lv.obj.FLAG.HIDDEN)
    
    data = arc.get_value()
    arc_label.set_text(str(arc.get_value()))
    arc.set_value(arc.get_value())
    arc.align_obj_to_angle(arc_label,0)   
    return(data)


#
### Display program and device info
#

t_info = lv.textarea( lv.scr_act() )
#t_info.set_style_text_font(lv.font_montserrat_16,0)
t_info.set_text( '      Reflow oven LK-014 (V1.0) \n \n' )

l_info = lv.label(lv.scr_act())
l_info.align(lv.ALIGN.CENTER,0,0)
l_info.set_text('')
#t_info.set_style_text_font(lv.font_montserrat_14,0)
print('\n  Reflow oven LK-014 (V1.0)' )
print('  SW: MicroPython '+ os.uname()[3]+ '\n  LVGL version: '+ lv.version_info() + ' '+ str(lv.version_major()) + '.' + str(lv.version_minor())+ '\n  HW: ' + os.uname()[4] + '\n')
t_info.add_text( '  SW: MicroPython '+ os.uname()[3]+ '\n  LVGL version: '+ lv.version_info() + ' '+ str(lv.version_major()) + '.' + str(lv.version_minor())+ '\n  HW: ' + os.uname()[4]  )

t_info.center()


#
###  Regulator for manual mode
#
def regulator_m(u_man,t):
    global  temp, delta_t, req_temp
      
    if delta_t < 0:
       u_man = 0.5*(t-int(temp))*t + (1 + 0.1*u_man*((t-int(temp))/10)) + 0.1*((t-int(temp))/10)*((delta_t*10)**2) + abs(int(t/200))*t + (t-46)*2    #(10 + int(u_man))*((250-int(req_temp))/50)
    else:
       u_man = 0.5*(t-int(temp))*t + (1 +0.1*u_man*((t-int(temp))/10)) - 0.1*((t-int(temp))/10)*((delta_t*10)**2) + abs(int(t/200))*t + (t-46)*2
    if (req_temp-int(temp-1)) <= 0:
       u_man = 0
    if u_man >= 1023:
       u_man = 1023
    if u_man <= 0:
       u_man = 0
    u_man = int(u_man)
    return(u_man)

#
### Ts samplig time of PID regulator
#

def PID_loop(PID_timer):
    global sp,pv, u
    u = pid_aw_updateControl(PID1,sp,pv,ubias=0.)
    return()


#
### function for set and plot temperature series
#
def plot_temp(timer):
    global is_start, x_time2, x_time, ser2, sp, r_time, a_time, r_temp, cnt_i, is_manual, x_range, is_plot, err_code,temp, req_time, cnt_run, run_manual, sp_m, cnt_err, u, u_man, delta_t, temp_b,h_text
    delta_t = temp - temp_b
    if temp <= sp  and is_start == 1 and is_manual == 0:
       cnt_err = cnt_err + 1
       if cnt_err > 180:
           print ("Heat components failed, REBOOT ... ")
           err_code = "   Heat components failed, REBOOT ...  "
    else:
       cnt_err = 0
    if req_temp >= int(temp)  and is_start == 1 and is_manual == 1:
       cnt_err = cnt_err + 1
       if cnt_err > 180:
           print ("Heat components failed, REBOOT ... ")
           err_code = "   Heat components failed, REBOOT ...  "
    else:
       cnt_err = 0

    if cnt_i == 1 and is_manual ==0:
       sp = int(r_temp[cnt_i-1]) - a_time*((int(r_temp[cnt_i-1]) - 50)/int(r_time[cnt_i-1]))
    else:
       if is_manual ==0:
          sp = int(r_temp[cnt_i-1]) - a_time*((int(r_temp[cnt_i-1]) - int(r_temp[cnt_i-2]))/int(r_time[cnt_i-1]))
    if cnt_i == len(r_time)-1 and a_time == 0:
       beep(freq=2637, time_b=1.2, duty=500)
       time.sleep(0.2)
       beep(freq=4186, time_b=0.01, duty=50)
       time.sleep(0.2)
       beep(freq=4186, time_b=0.01, duty=50)
       
    if temp > 50 or is_manual == 1:
     if x_time2 == x_time and is_start == 1 and is_manual == 0:                            # end heat and plot period
        a_time = 0
        cnt_i = 0
        if is_start == 1 :
           btn_start.set_style_bg_color(lv.palette_main(lv.PALETTE.BLUE),0)
           print ("Heat cyclus finished !")
        is_start = 0
        beep(freq=2637, time_b=1.2, duty=500)
     if is_plot == 1 or is_manual == 1:
        if run_manual == 1 and (int(temp) - req_temp ) < 0 and is_start == 1:
            a_time = a_time + 1
        else:
            run_manual = 0
        if is_manual == 1 and (req_time + a_time - 1) >= 0 and is_start == 1:
            l_header.set_text(h_text)
            u_man = regulator_m(u_man,req_temp)
        else:
            if is_manual == 1 and is_start:
               run_manual = 0
               u_man = 0
               sp = 50.                                   # set 50°C to finish chart
               is_start = is_plot = 0
               a_time = 0
               cnt_run = 0
               btn_start.set_style_bg_color(lv.palette_main(lv.PALETTE.BLUE),0)
               print ("Manual heat cyclus finished !")
               beep(freq=2637, time_b=1.2, duty=500)
               h_text=" Set temperature:  "+ str (req_temp) + "°C   time: "+str(int(req_time + a_time ))+ "sec."
               l_header.set_text(h_text)
        if a_time <= 0 and is_manual == 0 and is_start == 1:
            a_time = int(r_time[cnt_i])
            print("Heat time period: ", a_time," Temperature: ", r_temp[cnt_i] )
            cnt_i = cnt_i + 1
        if a_time <= 0 and is_manual == 0 and x_time2 == x_range:
            sp = 50.                                   # set 50°C to finish chart
        if  x_time2 >= x_range:   
            is_plot = 0
        if is_start == 1:
           a_time = a_time -1   
        x_time2 = x_time2 + 1
        chart.set_next_value2(ser2, x_time2, int(temp ))
        chart.refresh()
#    print(u_man, temp, req_temp)
    temp_b = temp
    return()


#
###   show actual temperature
#

text_temp = lv.label(scr)
text_temp.set_style_text_font(lv.font_montserrat_16,0)
text_temp.align(lv.ALIGN.TOP_LEFT,55,60)
text_temp.set_text( str(int(temp))+"°C")
text_temp


#
### set time interrupt for plot temerature
#
timer=Timer(2)
timer.init(period=1000,mode=Timer.PERIODIC, callback=plot_temp)

PID_timer =Timer(3)
PID_timer.init(period=100,mode=Timer.PERIODIC, callback=PID_loop)

#
###  define routine for temperature measurement
#
def measure_temp(temperature):
    #Read Celsius
    temperature = max31855.read_celsius()
    return(temperature)

#
### initialization of PID regulator
#
# pid structure creation
pid_buf=bytearray(101)   # size of PID_REGS is 101 bytes, 
PID1 = uctypes.struct(uctypes.addressof(pid_buf), PID_REGS, uctypes.LITTLE_ENDIAN)
pid_init0(PID1)

# lets change some params 
PID1.Kp = 1
PID1.Td = 1
PID1.Ti = 1
PID1.Tt = PID1.Ts            # 0.1
PID1.Umin = 0
PID1.Umax = 1023

pid_tune(PID1)            

# set P-I D:
PID1.CFG.Psel=  True ; PID1.CFG.Isel = True ;  PID1.CFG.Dsel = True   
# or pid.CFG_REG=0x03


    
    # and test settings
PID1.CFG.Modesel = False # True


#
###  speaker settings
#

def beep(freq, time_b, duty):
    O26 = PWM(Pin(26,Pin.OUT),freq , duty=512)
    time.sleep(time_b)
    O26.deinit()
    return()



#
###  modulate output 
#


O32 = PWM(Pin(32),freq = 1, duty=1023)

def set_output(u):
    if u > 1023:
       u = 1023
    if u <= 0:
       u = 0
    O32.duty((1023- u))
    return()

#
###   main cycle
#
  #Read Celsius
  #pv is process value
  #sp is set point requested value
  #u returned value from PID regulator
while err_code =="":
  pv= temp = float(measure_temp(temperature))  
  if int(temp) >=  300:
      err_code = h_text = "   Temperature sensor failed, REBOOT ...  "
  if r_set == 1:
     chart.remove_series(ser1)
     chart.remove_series(ser2)
     ser1 = chart.add_series(lv.palette_main(lv.PALETTE.RED), lv.chart.AXIS.PRIMARY_Y)
     ser2 = chart.add_series(lv.palette_main(lv.PALETTE.GREEN), lv.chart.AXIS.SECONDARY_Y)
     x_time=0
     chart.set_next_value2(ser1, x_time, 50)
     for i in range(0,len(r_time)):
         x_time = x_time + int(r_time[i])
         chart.set_next_value2(ser1, x_time, int(r_temp[i]))
     r_set = 0
  if is_manual == 1:
         h_text=" Set temperature:  "+ str (req_temp) + "°C   time: "+str(int(req_time + a_time ))+ "sec."
         x_time = req_time
         text_temp.set_text("")
         label_meter.set_text( str(int(temp))+"°C")
  else:
        text_temp.set_text( str(int(temp))+"°C")
  if is_start == 1 and is_manual == 0:
     set_output(int(u))
  if is_start == 1 and is_manual == 1:
     set_output(int(u_man))
  if is_start == 0:
      set_output(0)
  
  meter.set_indicator_value(indic_temp,int(temp))
  gc.collect()
  gc.mem_free()

  if is_manual == 1 and is_start == 0 and r_man == 1:
     req_time,req_temp = get_man_value(req_time,req_temp)
  
sp = 0
h_text = err_code
l_header.align(lv.ALIGN.CENTER,0,0)
l_header.set_text(h_text)
meter.add_flag(lv.obj.FLAG.HIDDEN)
chart.add_flag(lv.obj.FLAG.HIDDEN)
label_meter.add_flag(lv.obj.FLAG.HIDDEN)
label_X_axis.add_flag(lv.obj.FLAG.HIDDEN)
label_Y_axis.add_flag(lv.obj.FLAG.HIDDEN)
btn_start.set_style_bg_color(lv.palette_main(lv.PALETTE.GREY),0)
btn_stop.set_style_bg_color(lv.palette_main(lv.PALETTE.GREY),0)
btn_manual.set_style_bg_color(lv.palette_main(lv.PALETTE.GREY),0)
btn_set1.set_style_bg_color(lv.palette_main(lv.PALETTE.GREY),0)
btn_set2.set_style_bg_color(lv.palette_main(lv.PALETTE.GREY),0)
btn_set3.set_style_bg_color(lv.palette_main(lv.PALETTE.GREY),0)
print(err_code)
chart.refresh()
beep(freq=2637, time_b=3.2, duty=329)
time.sleep(15)
machine.reset()
